@@ -256,7 +256,7 @@ transaction_add_local_ref (FlatpakDir         *dir,
   g_autoptr(GError) local_error = NULL;
   g_autofree char *repo_checksum = NULL;
   const char *origin;
-  const char **subpaths;
+  g_autofree const char **subpaths = NULL;
 
   deploy_data = flatpak_dir_get_deploy_data (dir, ref, FLATPAK_DEPLOY_VERSION_ANY, NULL, &local_error);
   if (deploy_data == NULL)
@@ -290,7 +290,6 @@ flatpak_builtin_repair (int argc, char **argv, GCancellable *cancellable, GError
   g_autoptr(GPtrArray) refs = NULL;
   FlatpakDir *dir = NULL;
   g_autoptr(GHashTable) all_refs = NULL;
-  g_autoptr(GHashTable) invalid_refs = NULL;
   g_autoptr(GHashTable) object_status_cache = NULL;
   g_autoptr(FlatpakTransaction) transaction = NULL;
   OstreeRepo *repo;
@@ -343,8 +342,6 @@ flatpak_builtin_repair (int argc, char **argv, GCancellable *cancellable, GError
   object_status_cache = g_hash_table_new_full (ostree_hash_object_name, g_variant_equal,
                                                (GDestroyNotify) g_variant_unref, NULL);
 
-  invalid_refs = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
-
   /* Validate that the commit for each ref is available */
   if (!ostree_repo_list_refs (repo, NULL, &all_refs, cancellable, error))
     return FALSE;
@@ -358,6 +355,8 @@ flatpak_builtin_repair (int argc, char **argv, GCancellable *cancellable, GError
     g_autoptr(FlatpakDecomposed) ref = NULL;
     g_autofree char *origin = NULL;
 
+    i++;
+
     if (!ostree_parse_refspec (refspec, &remote, &ref_name, error))
       return FALSE;
 
@@ -391,7 +390,7 @@ flatpak_builtin_repair (int argc, char **argv, GCancellable *cancellable, GError
     if (flatpak_fancy_output ())
       g_print ("\033[A\r\033[K");
 
-    g_print (_("[%d/%d] Verifying %s…\n"), ++i, g_hash_table_size (all_refs), refspec);
+    g_print (_("[%d/%d] Verifying %s…\n"), i, g_hash_table_size (all_refs), refspec);
 
     status = fsck_commit (repo, checksum, object_status_cache);
     if (status != FSCK_STATUS_OK)
