@@ -894,16 +894,12 @@ print_perm_line (int        idx,
                  int        cols)
 {
   g_autoptr(GString) res = g_string_new (NULL);
-  g_autofree char *escaped_first_perm = NULL;
   int i;
 
-  escaped_first_perm = flatpak_escape_string (items->pdata[0], FLATPAK_ESCAPE_DEFAULT);
-  g_string_append_printf (res, "    [%d] %s", idx, escaped_first_perm);
+  g_string_append_printf (res, "    [%d] %s", idx, (char *) items->pdata[0]);
 
   for (i = 1; i < items->len; i++)
     {
-      g_autofree char *escaped = flatpak_escape_string (items->pdata[i],
-                                                        FLATPAK_ESCAPE_DEFAULT);
       char *p;
       int len;
 
@@ -912,10 +908,10 @@ print_perm_line (int        idx,
         p = res->str;
 
       len = (res->str + strlen (res->str)) - p;
-      if (len + strlen (escaped) + 2 >= cols)
-        g_string_append_printf (res, ",\n        %s", escaped);
+      if (len + strlen ((char *) items->pdata[i]) + 2 >= cols)
+        g_string_append_printf (res, ",\n        %s", (char *) items->pdata[i]);
       else
-        g_string_append_printf (res, ", %s", escaped);
+        g_string_append_printf (res, ", %s", (char *) items->pdata[i]);
     }
 
   g_print ("%s\n", res->str);
@@ -935,7 +931,7 @@ print_permissions (FlatpakCliTransaction *self,
   g_autoptr(GPtrArray) system_bus_talk = g_ptr_array_new_with_free_func (g_free);
   g_autoptr(GPtrArray) system_bus_own = g_ptr_array_new_with_free_func (g_free);
   g_autoptr(GPtrArray) tags = g_ptr_array_new_with_free_func (g_free);
-  FlatpakTablePrinter *printer;
+  g_autoptr(FlatpakTablePrinter) printer = NULL;
   int max_permission_width;
   int n_permission_cols;
   int i, j;
@@ -1018,7 +1014,6 @@ print_permissions (FlatpakCliTransaction *self,
     flatpak_table_printer_set_column_expand (printer, i, TRUE);
 
   flatpak_table_printer_print_full (printer, 0, cols, &table_rows, &table_cols);
-  flatpak_table_printer_free (printer);
 
   g_print ("\n\n");
 
@@ -1130,7 +1125,10 @@ transaction_ready_pre_auth (FlatpakTransaction *transaction)
 
   flatpak_table_printer_set_column_expand (printer, i, TRUE);
   if (!self->non_default_arch)
-    flatpak_table_printer_set_column_skip_unique (printer, i, TRUE);
+    {
+      flatpak_table_printer_set_column_skip_unique (printer, i, TRUE);
+      flatpak_table_printer_set_column_skip_unique_string (printer, i, flatpak_get_arch ());
+    }
   flatpak_table_printer_set_column_title (printer, i++, _("Arch"));
 
   flatpak_table_printer_set_column_expand (printer, i, TRUE);
