@@ -91,11 +91,11 @@ static FlatpakCommand commands[] = {
   { "create-usb", N_("Put applications or runtimes onto removable media"), flatpak_builtin_create_usb, flatpak_complete_create_usb },
 
   /* translators: please keep the leading newline and space */
-  { N_("\n Finding applications and runtimes") },
+  { N_("\n Find applications and runtimes") },
   { "search", N_("Search for remote apps/runtimes"), flatpak_builtin_search, flatpak_complete_search },
 
   /* translators: please keep the leading newline and space */
-  { N_("\n Running applications") },
+  { N_("\n Manage running applications") },
   { "run", N_("Run an application"), flatpak_builtin_run, flatpak_complete_run },
   { "override", N_("Override permissions for an application"), flatpak_builtin_override, flatpak_complete_override },
   { "make-current", N_("Specify default version to run"), flatpak_builtin_make_current_app, flatpak_complete_make_current_app },
@@ -817,11 +817,10 @@ flatpak_run (int      argc,
     check_environment ();
 
   /* Don't talk to dbus in enter, as it must be thread-free to setns, also
-     skip run/build/history for performance reasons (no need to connect to dbus). */
+     skip run/build for performance reasons (no need to connect to dbus). */
   if (g_strcmp0 (command->name, "enter") != 0 &&
       g_strcmp0 (command->name, "run") != 0 &&
-      g_strcmp0 (command->name, "build") != 0 &&
-      g_strcmp0 (command->name, "history") != 0)
+      g_strcmp0 (command->name, "build") != 0)
     polkit_agent = install_polkit_agent ();
 
   /* g_vfs_get_default can spawn threads */
