@@ -39,6 +39,7 @@
 #include "flatpak-error.h"
 #include "flatpak-oci-registry-private.h"
 #include "flatpak-progress-private.h"
+#include "flatpak-system-helper.h"
 #include "flatpak-utils-base-private.h"
 #include "flatpak-utils-private.h"
 
@@ -224,6 +225,7 @@ schedule_idle_callback (void)
 static FlatpakDir *
 dir_get_system (const char *installation,
                 pid_t       source_pid,
+                gboolean    no_interaction,
                 GError    **error)
 {
   FlatpakDir *system = NULL;
@@ -239,6 +241,7 @@ dir_get_system (const char *installation,
 
   flatpak_dir_set_source_pid (system, source_pid);
   flatpak_dir_set_no_system_helper (system, TRUE);
+  flatpak_dir_set_no_interaction (system, no_interaction);
 
   return system;
 }
@@ -403,56 +406,13 @@ handle_deploy (FlatpakSystemHelper   *object,
 
   g_debug ("Deploy %s %u %s %s %s", arg_repo_path, arg_flags, arg_ref, arg_origin, arg_installation);
 
-  system = dir_get_system (arg_installation, get_sender_pid (invocation), &error);
+  system = dir_get_system (arg_installation, get_sender_pid (invocation), (arg_flags & FLATPAK_HELPER_DEPLOY_FLAGS_NO_INTERACTION) != 0, &error);
   if (system == NULL)
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
       return G_DBUS_METHOD_INVOCATION_HANDLED;
     }
 
-  src_dir = g_path_get_dirname (arg_repo_path);
-  ongoing_pull = take_ongoing_pull_by_dir (src_dir);
-  if (ongoing_pull != NULL)
-    {
-      g_autoptr(GError) local_error = NULL;
-      uid_t uid;
-
-      /* Ensure that pull's uid is same as the caller's uid */
-      if (!get_connection_uid (invocation, &uid, &local_error))
-        {
-          g_dbus_method_invocation_return_gerror (invocation, local_error);
-          return G_DBUS_METHOD_INVOCATION_HANDLED;
-        }
-      else
-        {
-          if (ongoing_pull->uid != uid)
-            {
-              g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_FAILED,
-                                                     "Ongoing pull's uid(%d) does not match with peer uid(%d)",
-                                                     ongoing_pull->uid, uid);
-              return G_DBUS_METHOD_INVOCATION_HANDLED;
-            }
-        }
-
-      terminate_revokefs_backend (ongoing_pull);
-
-      if (!flatpak_canonicalize_permissions (AT_FDCWD,
-                                             arg_repo_path,
-                                             getuid() == 0 ? 0 : -1,
-                                             getuid() == 0 ? 0 : -1,
-                                             &local_error))
-        {
-          g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_FAILED,
-                                                 "Failed to canonicalize permissions of repo %s: %s",
-                                                 arg_repo_path, local_error->message);
-          return G_DBUS_METHOD_INVOCATION_HANDLED;
-        }
-
-      /* At this point, the cache-dir's repo is owned by root. Hence, any failure
-       * from here on, should always cleanup the cache-dir and not preserve it to be re-used. */
-      ongoing_pull->preserve_pull = FALSE;
-    }
-
   if ((arg_flags & ~FLATPAK_HELPER_DEPLOY_FLAGS_ALL) != 0)
     {
       g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS,
@@ -460,11 +420,57 @@ handle_deploy (FlatpakSystemHelper   *object,
       return G_DBUS_METHOD_INVOCATION_HANDLED;
     }
 
-  if (!g_file_query_exists (repo_file, NULL))
+  if (strlen (arg_repo_path) > 0)
     {
-      g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS,
-                                             "Path does not exist");
-      return G_DBUS_METHOD_INVOCATION_HANDLED;
+      if (!g_file_query_exists (repo_file, NULL))
+        {
+          g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS,
+                                                 "Path does not exist");
+          return G_DBUS_METHOD_INVOCATION_HANDLED;
+        }
+
+      src_dir = g_path_get_dirname (arg_repo_path);
+      ongoing_pull = take_ongoing_pull_by_dir (src_dir);
+      if (ongoing_pull != NULL)
+        {
+          g_autoptr(GError) local_error = NULL;
+          uid_t uid;
+
+          /* Ensure that pull's uid is same as the caller's uid */
+          if (!get_connection_uid (invocation, &uid, &local_error))
+            {
+              g_dbus_method_invocation_return_gerror (invocation, local_error);
+              return G_DBUS_METHOD_INVOCATION_HANDLED;
+            }
+          else
+            {
+              if (ongoing_pull->uid != uid)
+                {
+                  g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_FAILED,
+                                                         "Ongoing pull's uid(%d) does not match with peer uid(%d)",
+                                                         ongoing_pull->uid, uid);
+                  return G_DBUS_METHOD_INVOCATION_HANDLED;
+                }
+            }
+
+          terminate_revokefs_backend (ongoing_pull);
+
+          if (!flatpak_canonicalize_permissions (AT_FDCWD,
+                                                 arg_repo_path,
+                                                 getuid() == 0 ? 0 : -1,
+                                                 getuid() == 0 ? 0 : -1,
+                                                 &local_error))
+            {
+              g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_FAILED,
+                                                     "Failed to canonicalize permissions of repo %s: %s",
+                                                     arg_repo_path, local_error->message);
+              return G_DBUS_METHOD_INVOCATION_HANDLED;
+            }
+
+          /* At this point, the cache-dir's repo is owned by root. Hence, any failure
+           * from here on, should always cleanup the cache-dir and not preserve it to be re-used. */
+          ongoing_pull->preserve_pull = FALSE;
+        }
     }
 
   ref = flatpak_decomposed_new_from_ref (arg_ref, &error);
@@ -720,7 +726,7 @@ handle_cancel_pull (FlatpakSystemHelper   *object,
 
   g_debug ("CancelPull %s %u %s", arg_installation, arg_flags, arg_src_dir);
 
-  system = dir_get_system (arg_installation, get_sender_pid (invocation), &error);
+  system = dir_get_system (arg_installation, get_sender_pid (invocation), (arg_flags & FLATPAK_HELPER_CANCEL_PULL_FLAGS_NO_INTERACTION) != 0, &error);
   if (system == NULL)
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -778,7 +784,7 @@ handle_deploy_appstream (FlatpakSystemHelper   *object,
 
   g_debug ("DeployAppstream %s %u %s %s %s", arg_repo_path, arg_flags, arg_origin, arg_arch, arg_installation);
 
-  system = dir_get_system (arg_installation, get_sender_pid (invocation), &error);
+  system = dir_get_system (arg_installation, get_sender_pid (invocation), (arg_flags & FLATPAK_HELPER_DEPLOY_APPSTREAM_FLAGS_NO_INTERACTION) != 0, &error);
   if (system == NULL)
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -946,7 +952,7 @@ handle_uninstall (FlatpakSystemHelper   *object,
 
   g_debug ("Uninstall %u %s %s", arg_flags, arg_ref, arg_installation);
 
-  system = dir_get_system (arg_installation, get_sender_pid (invocation), &error);
+  system = dir_get_system (arg_installation, get_sender_pid (invocation), (arg_flags & FLATPAK_HELPER_UNINSTALL_FLAGS_NO_INTERACTION) != 0, &error);
   if (system == NULL)
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -999,7 +1005,7 @@ handle_install_bundle (FlatpakSystemHelper   *object,
 
   g_debug ("InstallBundle %s %u %s %s", arg_bundle_path, arg_flags, arg_remote, arg_installation);
 
-  system = dir_get_system (arg_installation, get_sender_pid (invocation), &error);
+  system = dir_get_system (arg_installation, get_sender_pid (invocation), (arg_flags & FLATPAK_HELPER_INSTALL_BUNDLE_FLAGS_NO_INTERACTION) != 0, &error);
   if (system == NULL)
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -1050,7 +1056,7 @@ handle_configure_remote (FlatpakSystemHelper   *object,
 
   g_debug ("ConfigureRemote %u %s %s", arg_flags, arg_remote, arg_installation);
 
-  system = dir_get_system (arg_installation, get_sender_pid (invocation), &error);
+  system = dir_get_system (arg_installation, get_sender_pid (invocation), (arg_flags & FLATPAK_HELPER_CONFIGURE_REMOTE_FLAGS_NO_INTERACTION) != 0, &error);
   if (system == NULL)
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -1125,7 +1131,7 @@ handle_configure (FlatpakSystemHelper   *object,
 
   g_debug ("Configure %u %s=%s %s", arg_flags, arg_key, arg_value, arg_installation);
 
-  system = dir_get_system (arg_installation, get_sender_pid (invocation), &error);
+  system = dir_get_system (arg_installation, get_sender_pid (invocation), (arg_flags & FLATPAK_HELPER_CONFIGURE_FLAGS_NO_INTERACTION) != 0, &error);
   if (system == NULL)
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -1191,7 +1197,7 @@ handle_update_remote (FlatpakSystemHelper   *object,
 
   g_debug ("UpdateRemote %u %s %s %s %s", arg_flags, arg_remote, arg_installation, arg_summary_path, arg_summary_sig_path);
 
-  system = dir_get_system (arg_installation, get_sender_pid (invocation), &error);
+  system = dir_get_system (arg_installation, get_sender_pid (invocation), (arg_flags & FLATPAK_HELPER_UPDATE_REMOTE_FLAGS_NO_INTERACTION) != 0, &error);
   if (system == NULL)
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -1270,7 +1276,7 @@ handle_remove_local_ref (FlatpakSystemHelper   *object,
 
   g_debug ("RemoveLocalRef %u %s %s %s", arg_flags, arg_remote, arg_ref, arg_installation);
 
-  system = dir_get_system (arg_installation, get_sender_pid (invocation), &error);
+  system = dir_get_system (arg_installation, get_sender_pid (invocation), (arg_flags & FLATPAK_HELPER_REMOVE_LOCAL_REF_FLAGS_NO_INTERACTION) != 0, &error);
   if (system == NULL)
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -1319,7 +1325,7 @@ handle_prune_local_repo (FlatpakSystemHelper   *object,
 
   g_debug ("PruneLocalRepo %u %s", arg_flags, arg_installation);
 
-  system = dir_get_system (arg_installation, get_sender_pid (invocation), &error);
+  system = dir_get_system (arg_installation, get_sender_pid (invocation), (arg_flags & FLATPAK_HELPER_PRUNE_LOCAL_REPO_FLAGS_NO_INTERACTION) != 0, &error);
   if (system == NULL)
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -1363,7 +1369,7 @@ handle_ensure_repo (FlatpakSystemHelper   *object,
 
   g_debug ("EnsureRepo %u %s", arg_flags, arg_installation);
 
-  system = dir_get_system (arg_installation, get_sender_pid (invocation), &error);
+  system = dir_get_system (arg_installation, get_sender_pid (invocation), (arg_flags & FLATPAK_HELPER_ENSURE_REPO_FLAGS_NO_INTERACTION) != 0, &error);
   if (system == NULL)
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -1402,7 +1408,7 @@ handle_run_triggers (FlatpakSystemHelper   *object,
 
   g_debug ("RunTriggers %u %s", arg_flags, arg_installation);
 
-  system = dir_get_system (arg_installation, get_sender_pid (invocation), &error);
+  system = dir_get_system (arg_installation, get_sender_pid (invocation), (arg_flags & FLATPAK_HELPER_RUN_TRIGGERS_FLAGS_NO_INTERACTION) != 0, &error);
   if (system == NULL)
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -1693,7 +1699,7 @@ handle_get_revokefs_fd (FlatpakSystemHelper   *object,
       return G_DBUS_METHOD_INVOCATION_HANDLED;
     }
 
-  system = dir_get_system (arg_installation, get_sender_pid (invocation), &error);
+  system = dir_get_system (arg_installation, get_sender_pid (invocation), (arg_flags & FLATPAK_HELPER_GET_REVOKEFS_FD_FLAGS_NO_INTERACTION) != 0, &error);
   if (system == NULL)
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -1790,7 +1796,7 @@ handle_update_summary (FlatpakSystemHelper   *object,
 
   g_debug ("UpdateSummary %u %s", arg_flags, arg_installation);
 
-  system = dir_get_system (arg_installation, get_sender_pid (invocation), &error);
+  system = dir_get_system (arg_installation, get_sender_pid (invocation), (arg_flags & FLATPAK_HELPER_UPDATE_SUMMARY_FLAGS_NO_INTERACTION) != 0, &error);
   if (system == NULL)
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -1836,7 +1842,7 @@ handle_generate_oci_summary (FlatpakSystemHelper   *object,
 
   g_debug ("GenerateOciSummary %u %s %s", arg_flags, arg_origin, arg_installation);
 
-  system = dir_get_system (arg_installation, get_sender_pid (invocation), &error);
+  system = dir_get_system (arg_installation, get_sender_pid (invocation), (arg_flags & FLATPAK_HELPER_GENERATE_OCI_SUMMARY_FLAGS_NO_INTERACTION) != 0, &error);
   if (system == NULL)
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -1947,6 +1953,8 @@ flatpak_authorize_method_handler (GDBusInterfaceSkeleton *interface,
               return FALSE;
             }
 
+          no_interaction = (flags & FLATPAK_HELPER_DEPLOY_FLAGS_NO_INTERACTION) != 0;
+
           /* These flags allow clients to "upgrade" the permission,
            * avoiding the need for multiple polkit dialogs when we first
            * update a runtime, then install the app that needs it.
@@ -1967,7 +1975,7 @@ flatpak_authorize_method_handler (GDBusInterfaceSkeleton *interface,
             is_install = TRUE;
           else
             {
-              g_autoptr(FlatpakDir) system = dir_get_system (installation, 0, &error);
+              g_autoptr(FlatpakDir) system = dir_get_system (installation, 0, no_interaction, &error);
 
               if (system == NULL)
                 {
@@ -1993,8 +2001,6 @@ flatpak_authorize_method_handler (GDBusInterfaceSkeleton *interface,
               else
                 action = "org.freedesktop.Flatpak.runtime-update";
             }
-
-          no_interaction = (flags & FLATPAK_HELPER_DEPLOY_FLAGS_NO_INTERACTION) != 0;
         }
 
       polkit_details_insert (details, "origin", origin);
@@ -2068,7 +2074,7 @@ flatpak_authorize_method_handler (GDBusInterfaceSkeleton *interface,
           g_autoptr(GError) sys_error = NULL;
           const char *name = NULL;
 
-          system = dir_get_system (installation, 0, &sys_error);
+          system = dir_get_system (installation, 0, no_interaction, &sys_error);
           if (system == NULL)
             {
               g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_FAILED,
@@ -2231,7 +2237,7 @@ on_bus_acquired (GDBusConnection *connection,
 
   if (!g_dbus_interface_skeleton_export (G_DBUS_INTERFACE_SKELETON (helper),
                                          connection,
-                                         "/org/freedesktop/Flatpak/SystemHelper",
+                                         FLATPAK_SYSTEM_HELPER_PATH,
                                          &error))
     {
       g_warning ("error: %s", error->message);
@@ -2407,7 +2413,7 @@ main (int    argc,
     flags |= G_BUS_NAME_OWNER_FLAGS_REPLACE;
 
   name_owner_id = g_bus_own_name (on_session_bus ? G_BUS_TYPE_SESSION  : G_BUS_TYPE_SYSTEM,
-                                  "org.freedesktop.Flatpak.SystemHelper",
+                                  FLATPAK_SYSTEM_HELPER_BUS_NAME,
                                   flags,
                                   on_bus_acquired,
                                   on_name_acquired,
